/**********************************************************
/* File Browser (Android)                                  *
/*                                                        *
/* Autor:   Roy Schneider                                 *
/*                                                        *
/* Version: 1.0.0                                         *
/*                                                        *
/* Copyright (c) 2012 Roy Schneider (all rights reserved) *
/*                                                        *
/**********************************************************/

// JAVA DOC INFORMATION
/**
@author     Roy Schneider
@see        <a href="http://www.overthehill.de">Author's Homepage</a>
*/

package overthehill.filebrowser_droid;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.TreeMap;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.KeyEvent;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;

/**
 * Klasse die den Dateibrowser implementiert. Bentigt zustzliche
 * die Klassen "AppFolderName" und "AppFileName" sowie die Dateien
 * "file_dialog_*.xml" aus dem "layout" und "menu" Ordner sowie
 * diverse Textitems aus der "strings.xml".
 */

public class AppFileBrowser extends ListActivity {

    /**
     * Eingebettete Aufzhlungsklasse, die die Dialogart beschreibt.
     */

    public class OperationMode {
        public static final int MODE_SAVE_AS = 0;    // "Save File"
        public static final int MODE_OPEN = 1;      // "Open File"
    }

    /**
     * Statics
     */

    /* Referenz auf die Instanz (Singleton)  */

    static AppFileBrowser m_AppFileBrowser = null;
    
    private static final String ITEM_KEY = "key";
    private static final String ITEM_IMAGE = "image";
    private static final String ROOT_FOLDER = "/";
    private static final String PARENT_FOLDER = "../";
    private static final String APP_LAST_FILE_PATH = "LastFilePath";

    public static final  String FORMAT_FILTER = "FORMAT_FILTER";
    public static final  String RESULT_PATH = "RESULT_PATH";
    public static final  String RESULT_NAME = "RESULT_NAME";
    public static final  String SELECTION_MODE = "SELECTION_MODE";

    public static final  int REQUEST_FOLDER_NAME = 5;
    public static final  int REQUEST_FILE_NAME = 6;

    static private String  m_parentPath;
    static private String  m_RenameFile;
    static private String  m_DeleteFile;
    static private File    m_selectedFile;
    static private boolean m_CancelDeleteFlag;
    static private boolean m_ExitFileBrowser;
    static private boolean m_PendingStillWorking;
    static private boolean m_UpdateFileList;

    /**
     * Attribute
     */

    private List<String> m_Path = null;
    private TextView m_FilePath;
    private EditText m_FileName;
    private ArrayList<HashMap<String, Object>> m_List;
    private int m_FolderItems;

    private Button m_OkButton;
    private Button m_ParentButton;
    private String m_CurrentPath = ROOT_FOLDER;

    private int m_OperationMode = OperationMode.MODE_SAVE_AS;

    private String[] m_formatFilter = null;

    private HashMap<String, Integer> m_lastPositions = new HashMap<String, Integer>();

    /* Thread und Handler fr Kommandoausfhrung
     * zur Trennung von UI-Thread und Workerthreads. */

    private Handler m_UpdateHandler;
    private Timer   m_UpdateTimer;

    /**
     * Methode: Wird aufgerufen, wenn das Activity erstellt wird.
     */

    @Override

    public void onCreate( Bundle savedInstanceState ) {
        String startPath;
        String DefFolderPath;
        String FolderPath;
        File StorageObj;

        /* Basisklasse aufrufen */

        super.onCreate( savedInstanceState );

        /* Instanz ablegen */
        
        m_AppFileBrowser = this;

        /* Workerthread Attribute */

        m_ExitFileBrowser = false;
        m_PendingStillWorking = false;
        m_UpdateFileList = false;

        m_UpdateHandler = null;
        m_UpdateTimer = null;

        /* Standard-Rckgabewert setzen */

        setResult( RESULT_CANCELED, getIntent() );

        /* Oberflchenlayout setzen */

        setContentView( R.layout.file_dialog_main );

        /* Referenzen zu den Steuerelementen ermitteln. */
        
        m_FilePath = ( TextView ) findViewById( R.id.path );
        m_FileName = ( EditText ) findViewById( R.id.filepathedit );

        /* Bei Auswahl der "RETURN"-Taste oder der "CENTER"-Taste, die Eingabe
         * auslesen und den Dialog automatisch beenden lassen. */

        m_FileName.setOnKeyListener( new EditText.OnKeyListener() {

            @Override

            public boolean onKey( View v, int keyCode, KeyEvent event ) {

                /* Schaltflche "OK"/"Speichern" des Besitzers freischalten/sperren. */

                //m_OkButton.setEnabled( ( m_FileName.length() > 0 ) ? true : false );

                /* Dialog beenden, wenn per Taste gefordert. */

                if ( ( keyCode == KeyEvent.KEYCODE_ENTER ) ||
                     ( keyCode == KeyEvent.KEYCODE_DPAD_CENTER ) ) {

                    EndDialog( true, true );
                    return ( true );
                }

                return false;
            }
        });
        
        /* Parameter an das Activity auslesen. */
        
        m_OperationMode = getIntent().getIntExtra( SELECTION_MODE, OperationMode.MODE_SAVE_AS );
        m_formatFilter  = getIntent().getStringArrayExtra( FORMAT_FILTER );

        /* Schaltflche "Abbruch" */

        final Button cancelButton = ( Button ) findViewById( R.id.ButtonCancel );
        cancelButton.setOnClickListener( new OnClickListener() {

            @Override
            public void onClick( View v ) {
                EndDialog ( false, false );
            }

        });

        /* Schaltflche "OK/Speichern" */

        m_OkButton = ( Button ) findViewById( R.id.ButtonOK );
        m_OkButton.setOnClickListener( new OnClickListener() {

            @Override
            public void onClick( View v ) {

                /* Pfad des neuen Unterordners als Zeichenkette an den Aufrufen zurckliefern. */

                EndDialog ( true, true );
            }
        } );

        /* Schaltflche "Neuer Ordner" */

        final Button folderButton = ( Button ) findViewById( R.id.ButtonFolder );
        folderButton.setOnClickListener( new OnClickListener() {

            @Override
            public void onClick( View v ) {

                /* Activity zur Ordnernamenseingabe erzeugen. */

                EnterNewFolderName();
            }
        } );

        /* Schaltflche "Hherer Ordner" */

        m_ParentButton = ( Button ) findViewById( R.id.ButtonParent );
        m_ParentButton.setOnClickListener( new OnClickListener() {

            @Override
            public void onClick( View v ) {

                /* In die nchst hherer Ordnerebene wechseln. */

                OpenParentFolder();
            }
        } );

        /* Soll der Dialog als "Datei ffnen..." arbeiten, dann die Schaltflche "OK"
         * erst freigeben, wenn ein Item ausgewhlt wurde. */
        
        m_OkButton.setText( ( m_OperationMode == OperationMode.MODE_OPEN ) ? R.string.StrOpen : R.string.StrSave );
        //m_OkButton.setEnabled( false );

        /* Ausgangspfad setzen. */

        if ( m_selectedFile != null ) {

            /* Ist bereits ein Pfad ausgewhlt? */

            startPath = m_selectedFile.getPath();
        } else {

            /* Pfad zum Massenspeicher ermitteln. */

            final String StorageState = Environment.getExternalStorageState();

            /* Standardmig versuchen den SD-Karten-Root verwenden. */

            StorageObj = null;

            if ( Environment.MEDIA_MOUNTED.equals( StorageState ) ) {
                try {
                    StorageObj = Environment.getExternalStorageDirectory();
                } catch ( Exception e ) {
                }
            }        

            /* Ab API Level 8 */
            /*
            if ( StorageObj == null ){
                try {
                    StorageObj = Environment.getExternalStoragePublicDirectory( Environment.DIRECTORY_DOWNLOADS )
                } catch ( Exception e ) {
                }
            }
            */

            /* Pfadnamen als Zeichenkette ermitteln. */

            if ( StorageObj != null ) {
                DefFolderPath = StorageObj.toString();
            } else {
                DefFolderPath = ROOT_FOLDER;
            }

            FolderPath = DefFolderPath;

            // Vom Anwender zuletzt ausgewhlten Speicherort einlesen.

            try {
                SharedPreferences PrefsObj = getPreferences( 0 );
                if ( PrefsObj != null ) {
                    FolderPath = PrefsObj.getString( APP_LAST_FILE_PATH, DefFolderPath );

                    /* Prfen, ob der zuletzt gespeicherte Pfad existiert. */

                    if ( FolderPath.length() > 0 ) {
                        if ( ! ( new File ( FolderPath ).exists() ) ) {
                            FolderPath = DefFolderPath;
                        }
                    }
                }
            } catch ( Exception e ) {
            }

            startPath = FolderPath;
        }

        if ( startPath == null ) {
            startPath = ROOT_FOLDER;
        }

        m_selectedFile = new File( startPath );

        /* Listview mit dem Kontextmen registrieren. */

        registerForContextMenu ( getListView() );

        /* Verzeichnis einlesen. */

        getDir( startPath );
    }


    /**
     * Methode: Wird aufgerufen, um das Activity (Dialog) zu beenden.
     * 
     * @param boolean bok     true, um OK und false um CANCELED zurck zuliefern.
     * @param boolean bcheck  true, wenn geprft werden soll, ob die Datei bereits existiert.
     */

    protected final void EndDialog( boolean bok, boolean bcheck ) {
        SharedPreferences PrefsObj;
        SharedPreferences.Editor PrefsEditor;

        /* Workerthread abbrechen/beenden */

        m_ExitFileBrowser = true;

        if ( m_UpdateTimer != null ) {
            m_UpdateTimer.cancel();
        }

        /* Activity beenden */

        if ( bok ) {

            /* Die Auswahl zurckliefern. */

            final String Filename = m_FileName.getEditableText().toString();

            Filename.trim();

            if ( Filename.length() <= 0 ) {

                new AlertDialog.Builder( this ).setIcon( android.R.drawable.ic_menu_info_details )
                .setTitle( R.string.StrFileIOActivity )
                .setMessage( R.string.StrEnterFilename )
                .setPositiveButton( getText( R.string.StrOK ), new DialogInterface.OnClickListener() {

                    @Override

                    public void onClick( DialogInterface dialog, int which ) {
                    }
                }).show();

                return;
            }

            /* Soll die Auswahl geprft werden und es handelt
             * sich um ein "Speichern unter..." Activity. */

            if ( ( bcheck ) && ( m_OperationMode == OperationMode.MODE_SAVE_AS ) ) {

                /* Prfen, ob die Datei bereits existiert. */

                File nwfile = new File ( m_CurrentPath + "/" + Filename );

                if ( nwfile != null ) {
                    if ( nwfile.exists() ) {

                        /* Sicherheitsabfrage */

                        new AlertDialog.Builder( this ).setIcon( android.R.drawable.ic_menu_help )
                        .setTitle( R.string.StrNotify )
                                .setMessage( getText( R.string.StrOverwriteSure ) + "\n\n" + getText ( R.string.StrFileName ) + " \"" + Filename + "\"" )
                                .setNegativeButton( getText( R.string.StrNo ), null )
                                .setPositiveButton( getText( R.string.StrYes ), new DialogInterface.OnClickListener() {

                                    @Override

                                    public void onClick( DialogInterface dialog, int which ) {

                                        /* Vorhandene Datei berschreiben -> Activity (Dialog) zu beenden lassen. */

                                        EndDialog ( true, false /* Keine erneute Prfung ob die Datei existiert. */ );
                                    }
                                }).show();
                    } else {

                        /* Datei existiert nicht -> Activity (Dialog) zu beenden lassen. */

                        EndDialog ( true, false /* Keine erneute Prfung ob die Datei existiert. */ );
                    }
                }
            } else {
                getIntent().putExtra( RESULT_PATH, m_CurrentPath );
                getIntent().putExtra( RESULT_NAME, Filename );
    
                /* Vom Anwender ausgewhlten Speicherort ablegen. */
    
                try {
                    PrefsObj = getPreferences( 0 );
    
                    if ( PrefsObj != null ) {
                        PrefsEditor = PrefsObj.edit();
    
                        if ( PrefsEditor != null ) {
                            PrefsEditor.putString( APP_LAST_FILE_PATH, m_selectedFile.getPath() );
                            PrefsEditor.commit();
                        }
                    }
                } catch ( Exception e ) {
                }
    
                /* Erfolg signalisieren. */
                        
                setResult( RESULT_OK, getIntent() );
    
                m_selectedFile = null;
    
                InputMethodManager imm = ( InputMethodManager ) getSystemService( Context.INPUT_METHOD_SERVICE );
                imm.hideSoftInputFromWindow ( m_FileName.getWindowToken(), 0);

                /* Activity (Dialog) schlieen. */
    
                finish();
            }
        } else {

            /* Keine Auswahl zurckliefern. */

            setResult( RESULT_CANCELED, getIntent() );

            m_selectedFile = null;

            /* Activity (Dialog) schlieen. */

            finish();
        }
    }

    /**
     * Methode: Wird aufgerufen, um den Dialog fr die Namenseingabe
     *          einer umzubennenden Datei zu ffnen.
     * 
     * @param boolean bfolder  true, wenn das  Dateiobjekt ein Ordner ist.
     * @param String FName     Aktueller Dateiname
     */

    private void EnterFileName( boolean bfolder, String FName ) {
        
        AppFileName.SetFolderMode ( bfolder );
        AppFileName.SetFilename( FName );

        Intent serverIntentFileName = new Intent( this, AppFileName.class );
        startActivityForResult( serverIntentFileName, REQUEST_FILE_NAME );
    }

    /**
     * Methode: Wird aufgerufen, um den Dialog fr die Namenseingabe
     *          eines neu zu erstellenden Ordners zu ffnen.
     */

    private void EnterNewFolderName() {
        
        AppFileName.SetFolderMode ( true );
        AppFileName.SetFilename( "" );

        Intent serverIntentFileName = new Intent( this, AppFileName.class );
        startActivityForResult( serverIntentFileName, REQUEST_FOLDER_NAME );
    }

    /**
     * Methode: Wird aufgerufen, um das Verzeichnis einlesen zu lassen
     *          und die Auswahl zu aktualisieren.
     * 
     * @param String dirPath  Referenz auf den Pfadnamen
     */
    
    final private void getDir( String dirPath ) {
        boolean useAutoSelection;

        useAutoSelection = dirPath.length() < m_CurrentPath.length();

        Integer position = m_lastPositions.get( m_parentPath );

        getDirImpl( dirPath );

        if ( ( position != null ) && ( useAutoSelection ) ) {
            getListView().setSelection( position );
        }

        /* Rootebene erreicht? */

        m_ParentButton.setEnabled( ( m_CurrentPath.equals( ROOT_FOLDER ) ) ? false : true );
    }

    /**
     * Methode: Wird aufgerufen, um das Verzeichnis einlesen.
     * 
     * @param String dirPath  Referenz auf den Pfadnamen
     */

    final private void getDirImpl( final String dirPath ) {
        int i;
        File f;
        File[] files;
        String dirName;
        boolean badd;

        m_CurrentPath = dirPath;

        final List<String> item = new ArrayList<String>();

        m_Path = new ArrayList<String>();
        m_List = new ArrayList<HashMap<String, Object>>();

        files = null;

        f = new File( m_CurrentPath );
        if ( f != null ) {
            files = f.listFiles();
        }

        if ( files == null ) {
            m_CurrentPath = ROOT_FOLDER;

            f = new File( m_CurrentPath );
            files = f.listFiles();
        }

        m_FilePath.setText( getText( R.string.StrPath ) + ": " + m_CurrentPath );

        /* "Root" und "Parent" Item einfgen */
        
        if ( ! m_CurrentPath.equals( ROOT_FOLDER ) ) {

            m_FolderItems = 0;

            /* Root ("/") */

            /*
            item.add( ROOT_FOLDER );
            addItem( ROOT_FOLDER, R.drawable.folder );
            m_Path.add( ROOT_FOLDER );
            m_FolderItems++;
            */

            /* Parent ("../") */

            item.add( PARENT_FOLDER );
            addItem( PARENT_FOLDER, R.drawable.folder );
            m_Path.add( f.getParent() );
            m_FolderItems++;

            m_parentPath = f.getParent();
        }

        /* Map fr Ordner und ihre Pfad anlegen. */

        TreeMap<String, String> dirsMap = new TreeMap<String, String>();
        TreeMap<String, String> dirsPathMap = new TreeMap<String, String>();

        /* Map fr Dateien und ihre Pfad anlegen. */

        TreeMap<String, String> filesMap = new TreeMap<String, String>();
        TreeMap<String, String> filesPathMap = new TreeMap<String, String>();

        /* Verzeichnis einlesen */

        for ( File file : files ) {

            /* Nchstes zurckgeliefertes Objekt ist ein Verzeichnis? */

            if ( file.isDirectory() ) {

                /* Verzeichnis */

                dirName = file.getName();

                if ( ! dirName.startsWith(".") ) {
                    dirsMap.put( dirName, dirName );
                    dirsPathMap.put( dirName, file.getPath() );
                }
            } else {

                /* Datei */

                final String fileName = file.getName();

                /* Dateifilter (Dateityp) anwenden. */

                if ( m_formatFilter != null ) {
                    final String fileNameLwr = fileName.toLowerCase();

                    badd = false;

                    for ( i = 0; i < m_formatFilter.length; i++ ) {
                        final String formatLwr = m_formatFilter[i].toLowerCase();

                        if ( fileNameLwr.endsWith( formatLwr ) ) {
                            badd = true;
                            break;
                        }
                    }
                } else {
                    badd = true;
                }

                if ( badd ) {
                    filesMap.put( fileName, fileName );
                    filesPathMap.put( fileName, file.getPath() );
                }
            }
        }

        /* Erst alle Ordner und dann alle Dateien der Liste hinzufgen. */ 

        final String emptyString = "";

        item.addAll( dirsMap.tailMap( emptyString ).values() );
        item.addAll( filesMap.tailMap( emptyString ).values() );

        m_Path.addAll( dirsPathMap.tailMap( emptyString ).values() );
        m_Path.addAll( filesPathMap.tailMap( emptyString ).values() );

        /* Layout fr Zeile im Listview festlegen. */

        SimpleAdapter fileList = new SimpleAdapter( this, m_List, R.layout.file_dialog_row, new String[] { ITEM_KEY, ITEM_IMAGE }, new int[] { R.id.rowtext, R.id.rowimage } );

        for ( String dir : dirsMap.tailMap( emptyString ).values() ) {
            addItem( dir, R.drawable.folder );
        }

        for ( String file : filesMap.tailMap( emptyString ).values() ) {
            addItem( file, R.drawable.file );
        }

        /* Listview benachrichtigen, dass sich sein Inhalt gendert hat. */

        fileList.notifyDataSetChanged();

        setListAdapter( fileList );
    }

    /**
     * Methode: Wird aufgerufen, um einen Eintrag dem Listview hinzuzufgen.
     * 
     * @param String fileName  Datei-/Verzeichnisname
     * @param int imageId      Image-Id (fr Verzeichnisse)
     */

    private final void addItem( String fileName, int imageId ) {
        HashMap<String, Object> item = new HashMap<String, Object>();

        item.put( ITEM_KEY, fileName );
        item.put( ITEM_IMAGE, imageId );

        m_List.add( item );
    }

    /**
     * Methode: Wird aufgerufen, wenn auf ein Item im Listview geklickt wurde.
     * 
     * @param ListView l    Referenz auf den Listview
     * @param View v        Besitzende Ansicht
     * @param int position  Index des Items
     * @param long id       Id des items
     */

    @Override

    protected void onListItemClick( ListView l, View v, int position, long id ) {

        File file = new File( m_Path.get( position ) );

        /* Ist die Auswahl ein Verzeichnis? */

        if ( file.isDirectory() ) {

            /* Prfen, ob der Inhalt des Verzeichnisses gelesen werden darf. */

            if ( file.canRead() ) {
                m_lastPositions.put( m_CurrentPath, position );

                getDir( m_Path.get( position ) );

                m_selectedFile = file;
            } else {

                /* Kann der Inhalt des Verzeichnisses nicht aufgelistet
                 * werden (geschtzt) dann einen Hinweis ausgeben. */

                new AlertDialog.Builder( this ).setIcon( android.R.drawable.ic_dialog_alert )
                .setTitle( R.string.StrNotify )
                        .setMessage( getText( R.string.StrFolderProtected ) + "\n\n" + getText ( R.string.StrFolderName ) + " \"" + file.getName() + "\"" )
                        .setPositiveButton( getText( R.string.StrOK ), new DialogInterface.OnClickListener() {

                            @Override
                            public void onClick( DialogInterface dialog, int which ) {
                            }
                        }).show();
            }
        } else {

            if ( file != null ) {
                m_FileName.setText( file.getName() );
            }

            m_OkButton.setEnabled ( ( m_FileName.length() > 0 ) ? true : false );
        }
    }


    /**
     * Methode: Wird aufgerufen, wenn eine Taste im View gedrckt wurde.
     * 
     * @param int keyCode     Tastencode
     * @param KeyEvent event  Tastenereignis
     * 
     * @return boolean  true, wenn die Taste verarbeitet wurde.
     */

    @Override

    public boolean onKeyDown( int keycode, KeyEvent event ) {

        /* "Back"-Button behandeln. */
        
        if ( keycode == KeyEvent.KEYCODE_BACK ) {
            EndDialog( false, false );
            
            return ( true );
        }

        /* Wurde eine Taste gedrckt, die nicht von der Anwendung verarbeitet werden soll,
         * dann die Basisklasse aufrufen und die Taste durch das System verarbeiten lassen. */

        return ( super.onKeyDown( keycode, event ) );
    }

    /**
     * Methode: Wird aufgerufen, wenn eine Benutzeraktivitt beendet wurde.
     * 
     * @param int requestCode   Code des Requests
     * @param int resultCode    Ergebniscode
     * @param Intent data       Benutzerdefinierte Daten
     */

    @Override

    public void onActivityResult( int requestCode, int resultCode, Intent data ) {

        switch ( requestCode ) {

        /* Anwender hat den Namen fr den neuen Ordner eingegeben. */

        case REQUEST_FOLDER_NAME:
             if ( resultCode == Activity.RESULT_OK ) {

                 /* Ordnernamen bernehmen. */

                 String EditName = data.getExtras().getString( AppFileName.EXTRA_FILE_NAME );

                 if ( EditName.length() > 0 ) {

                     /* Ordnerpfad zusammensetzen */

                     File nwfolder = new File ( m_selectedFile.getPath() + "/" + EditName.toString() );

                     /* Prfen, ob der neue Ordner bereits existiert. */

                     if ( nwfolder.exists() ) {

                         /* Einen Hinweis ausgeben, dass die Datei oder der Ordner nicht umbenannt
                          * werden konnte, weil ein Objekt gleichen Namens bereits existiert. */

                         final boolean bdir = nwfolder.isDirectory();

                         new AlertDialog.Builder( this ).setIcon( android.R.drawable.ic_dialog_alert )
                                 .setTitle( R.string.StrNewFolder )
                                 .setMessage( getText( bdir ? R.string.StrCreateFolderAlreadyExistsFolder : R.string.StrCreateFolderAlreadyExistsFile ) + "\n\n" + getText ( bdir ? R.string.StrFolderName : R.string.StrFileName ) + " \"" + nwfolder.getName() + "\"" )
                                 .setPositiveButton( getText( R.string.StrOK ), new DialogInterface.OnClickListener() {

                                     @Override

                                     public void onClick( DialogInterface dialog, int which ) {
                                     }
                                 }).show();
                     } else {

                         /* Ordner anlegen */

                         if ( nwfolder.mkdir() == true ) {

                             /* Verzeichnisliste aktualisieren. */

                             getDir ( m_CurrentPath );
                         } else {

                             /* Kann der neue Ordner im Verzeichnis nicht angelegt
                              * werden (geschtzt) dann einen Hinweis ausgeben. */

                             new AlertDialog.Builder( this ).setIcon( android.R.drawable.ic_dialog_alert )
                                     .setTitle( R.string.StrNewFolder )
                                     .setMessage( getText( R.string.StrFolderCreateFailed ) + "\n\n" + getText ( R.string.StrFolderName ) + " \"" + nwfolder.getName() + "\"" )
                                     .setPositiveButton( getText( R.string.StrOK ), new DialogInterface.OnClickListener() {

                                         @Override
                                         public void onClick( DialogInterface dialog, int which ) {
                                         }
                                     }).show();
                         }
                     }
                 }
             }
        break;

        /* Anwender hat den neuen Namen einen Datei/Ordner eingegeben. */

        case REQUEST_FILE_NAME:
             if ( resultCode == Activity.RESULT_OK ) {

                 /* Ordnernamen bernehmen. */

                 String EditName = data.getExtras().getString( AppFileName.EXTRA_FILE_NAME );

                 if ( EditName.length() > 0 ) {

                     File nwfile = new File ( m_selectedFile.getPath() + "/" + EditName.toString() );

                     if ( nwfile != null ) {
                         File olfile = new File ( m_RenameFile );

                         if ( olfile != null ) {

                             /* Prfen, ob die Zieldatei/-ordner bereits existiert. */

                             if ( nwfile.exists() ) {

                                 /* Einen Hinweis ausgeben, dass die Datei oder der Ordner nicht umbenannt
                                  * werden konnte, weil ein Objekt gleichen Namens bereits existiert. */

                                 final boolean bdir = nwfile.isDirectory();
                                 
                                 new AlertDialog.Builder( this ).setIcon( android.R.drawable.ic_dialog_alert )
                                         .setTitle( R.string.StrRename )
                                         .setMessage( getText( bdir ? R.string.StrRenameFolderAlreadyExists : R.string.StrRenameFileAlreadyExists ) + "\n\n" + getText ( bdir ? R.string.StrFolderName : R.string.StrFileName ) + " \"" + nwfile.getName() + "\"" )
                                         .setPositiveButton( getText( R.string.StrOK ), new DialogInterface.OnClickListener() {

                                             @Override

                                             public void onClick( DialogInterface dialog, int which ) {
                                             }
                                         }).show();
                             } else {

                                 /* Parameter an das Activity auslesen. */

                                 if ( olfile.renameTo( nwfile ) == true ) {

                                     /* Verzeichnisliste aktualisieren. */

                                     getDir ( m_CurrentPath );
                                 } else {

                                     /* Kann die Datei/Ordner nicht umbenannt werden, dann
                                      * einen Hinweis ausgeben. */

                                     final boolean bdir = olfile.isDirectory();

                                     new AlertDialog.Builder( this ).setIcon( android.R.drawable.ic_dialog_alert )
                                             .setTitle( R.string.StrRename )
                                             .setMessage( getText( bdir ? R.string.StrRenameFolderFailed : R.string.StrRenameFileFailed ) + "\n\n" + getText ( bdir ? R.string.StrFolderName : R.string.StrFileName ) + " \"" + olfile.getName() + "\"" )
                                             .setPositiveButton( getText( R.string.StrOK ), new DialogInterface.OnClickListener() {

                                                 @Override

                                                 public void onClick( DialogInterface dialog, int which ) {
                                                 }
                                             }).show();
                                 }
                             }
                         }
                     }
                 }
             }
        break;

        default:
            break;
        }
    }

    /**
     * Methode: Wird aufgerufen, wenn ein Kontextmen geffnet wird.
     * 
     * @param ContextMenu menu          Referenz auf das angeforderte Kontextmen.
     * @param View v                    Refrenz auf den View der das Kontextmen anfordert.
     * @param ContextMenuInfo menuInfo  Refrenz auf die Kontextmenbeschreibung.
     */

    @Override

    public void onCreateContextMenu( ContextMenu menu, View v, ContextMenuInfo menuInfo ) {
        long id;

        /* Basisklasse aufrufen */

        super.onCreateContextMenu( menu, v, menuInfo );

        /* Ermitteln auf welches Item geklickt wurde. */

        AdapterView.AdapterContextMenuInfo info = ( AdapterView.AdapterContextMenuInfo ) menuInfo;

        if ( info != null ) {
            id = info.id;

            /* Prfen, ob dass Root oder Parent Item ausgewhlt wurde. */

            if ( id < m_FolderItems ) {
                return;
            }
        }

        /* Menbeschreibung (XML) holen. */

        MenuInflater inflater = getMenuInflater();

        if ( inflater != null ) {
            inflater.inflate( R.menu.file_dialog_context_menu, menu );
        }
    }

    /**
     * Methode: Wird aufgerufen, wenn ein Item aus einem Kontextmen ausgewhlt wird.
     * 
     * @param MenuItem item  Refrenz auf das ausgewhlte Menitem.
     * 
     * @return boolean  true, wenn das Item verarbeitet wurde.
     */

    @Override

    public boolean onContextItemSelected( MenuItem item ) {
        long id;

        /* Item und dessen Id ermitteln, dass das Kontextmen ausgelst hat. */

        AdapterContextMenuInfo info = ( AdapterContextMenuInfo ) item.getMenuInfo();

        if ( info != null ) {
            id = info.id;

            /* Prfen, ob dass Root oder Parent Item ausgewhlt wurde. */

            if ( id < m_FolderItems ) {
                return ( false );
            }

            /* Im Kontextmen ausgewhlte Funktion ausfhren. */

            if ( info != null ) {
                switch ( item.getItemId() ) {
                    case R.id.file_dialog_rename:

                        /* Ausgewhlte Datei/Ordner umzubenennen. */

                        RenameFile( id );
                        return ( true );

                    case R.id.file_dialog_delete:

                        /* Ausgewhlte Datei/Ordner lschen. */

                        DeleteFile( id );
                        return ( true );

                    default:
                        break;
                }
            }
        }

        /* Basisklasse aufrufen */

        return ( super.onContextItemSelected( item ) );
    }

    /**
     * Methode: Wird aufgerufen, um eine im Listview ausgewhlte
     *          Datei/Ordner zu lschen.
     * 
     * @param long id  Id des Listviewitems
     */

    private void DeleteFile( long id ) {
        String FileName;

        FileName = m_Path.get( ( int ) id );

        if ( FileName != null ) {

            /* Namen der zu lschenden Datei ablegen. */

            m_DeleteFile = FileName;

            /* Sicherheitsabfrage */

            final boolean bdir = new File ( FileName ).isDirectory();

            new AlertDialog.Builder( this ).setIcon( android.R.drawable.ic_menu_help )
            .setTitle( R.string.StrNotify )
                    .setMessage( getText( bdir ? R.string.StrDeleteSureFolder : R.string.StrDeleteSureFile ) + "\n\n" + getText ( bdir ? R.string.StrFolderName : R.string.StrFileName ) + " \"" + ( String ) m_List.get( ( int ) id ).get( ITEM_KEY ) + "\"" )
                    .setNegativeButton( getText( R.string.StrNo ), null )
                    .setPositiveButton( getText( R.string.StrYes ), new DialogInterface.OnClickListener() {

                        @Override

                        public void onClick( DialogInterface dialog, int which ) {

                            File fl = new File ( m_DeleteFile );

                            if ( fl != null ) {

                                /* Einzelne Datei/Ordner lschen lassen. */

                                if ( fl.delete() == true ) {

                                    /* Verzeichnisliste aktualisieren. */

                                    getDir ( m_CurrentPath );
                                } else {

                                    /* Verzeichnis samt Unterverzeichnisse von
                                     * einem Workerthread lschen lassen. */

                                    if ( DeleteRecursiveWorker( fl ) == false ) {

                                        /* Kann die Datei/Ordner nicht gelscht werden, dann
                                         * einen Hinweis ausgeben. */

                                        new AlertDialog.Builder( m_AppFileBrowser ).setIcon( android.R.drawable.ic_dialog_alert )
                                                .setTitle( R.string.StrDelete )
                                                .setMessage( getText( fl.isDirectory() ? R.string.StrDeleteFolderFailed : R.string.StrDeleteFileFailed ) + "\n\n" + getText ( fl.isDirectory() ? R.string.StrFolderName : R.string.StrFileName ) + " \"" + fl.getName() + "\"" )
                                                .setPositiveButton( getText( R.string.StrOK ), new DialogInterface.OnClickListener() {

                                                    @Override
                                                    public void onClick( DialogInterface dialog, int which ) {
                                                    }
                                                }).show();
                                    }
                                }
                            }
                        }
                    }).show();
        }
    }


    /**
     * Methode: Wird aufgerufen, um ein Verzeichnis samt Unterverzeichnisse 
     *          von einem Workerthread lschen zu lassen.
     * 
     * @param File fileobj  Dateiobjekt (Datei/Ordner)
     * 
     * @return boolean  true/false  Erfolg/Fehler
     */

    boolean DeleteRecursiveWorker( File fileobj ) {
        final File worker_fileobj = fileobj;
        boolean bsuccesss;

        bsuccesss = true;

        /* Handler fr Aktualisierung nach Lschung der
         * Dateien durch den Workerthread einrichten,
         * wenn er nicht schon existiert. */

        m_PendingStillWorking = false;
        m_UpdateFileList = false;

        if ( m_UpdateHandler == null ) {
            m_UpdateHandler = new Handler();

            if ( m_UpdateHandler != null ) {
                m_UpdateTimer = new Timer();

                if ( m_UpdateTimer != null ) {
                    m_UpdateTimer.schedule( new UpdateTimeTask(), 250, 250 );
                }
            }
        }

        /* Lschen-Dialog erstellen. */

        m_CancelDeleteFlag = false;

        final ProgressDialog DeleteDlg = new ProgressDialog( this );

        try {
            DeleteDlg.setTitle( R.string.StrDeleting );
            DeleteDlg.setMessage( worker_fileobj.getName() );

            /* Abbruch-Schaltflche hinzufgen. */

            DeleteDlg.setButton( getText( R.string.StrCancel ), new DialogInterface.OnClickListener() {
                public void onClick( DialogInterface dialog, int which )  {

                    /* Lschvorgang abbrechen lassen. */

                    m_CancelDeleteFlag = true;
                }
            } );

            /* Lschen-Dialog anzeigen */

            DeleteDlg.show();

        } catch ( Exception e ) {
            bsuccesss = false;
        }

        /* Workerthread */

        new Thread( new Runnable() {
            public void run() {
                long tmespn;
                boolean bdelete;

                /* Sicherstellen, dass das Fenster mindestens eine
                 * Sekunde sichtbar ist. */

                tmespn = System.currentTimeMillis() + 1000;
                
                /* Inhalt rekursiv lschen. */

                bdelete = DeleteRecursive ( worker_fileobj );

                /* Kurze Pause, damit das Fenster auch sichtbar war. */

                tmespn = ( System.currentTimeMillis() - tmespn );

                if ( tmespn < 0 ) {
                    try {
                        Thread.sleep( -tmespn );
                    } catch ( InterruptedException e1 ) {
                        e1.printStackTrace();
                    }
                }

                /* Flag setzen, um Dateiliste zu aktualisieren. */

                m_UpdateFileList = true;

                /* Lschen-Dialog zerstren. */

                try {
                    DeleteDlg.dismiss();
                } catch ( Exception e ) {
                }

                /* Abbruch-Flag zurcksetzen */
                
                m_CancelDeleteFlag = false;

                /* Fehler? */
                
                if ( ! bdelete ) {

                    /* Kann die Datei/Ordner nicht gelscht werden, dann
                     * einen Hinweis ausgeben. */

                    new AlertDialog.Builder( m_AppFileBrowser ).setIcon( android.R.drawable.ic_dialog_alert )
                            .setTitle( R.string.StrDelete )
                            .setMessage( getText( worker_fileobj.isDirectory() ? R.string.StrDeleteFolderFailed : R.string.StrDeleteFileFailed ) + "\n\n" + getText ( worker_fileobj.isDirectory() ? R.string.StrFolderName : R.string.StrFileName ) + " \"" + worker_fileobj.getName() + "\"" )
                            .setPositiveButton( getText( R.string.StrOK ), new DialogInterface.OnClickListener() {

                                @Override
                                public void onClick( DialogInterface dialog, int which ) {
                                }
                            }).show();
                }
            }
        }).start();

        return ( bsuccesss );
    }

    /**
     * GUI-Timer-Task
     */

    class UpdateTimeTask extends TimerTask {

        /**
         * Methode, die zyklisch in der beim Erstellen festgelegten
         * Zeit aufgerufen wird.
         */

        public void run() {

            if ( ! m_ExitFileBrowser ) {

                /* Flag testen, dass signalisiert, das der GUI-Updatehandler luft. */

                if ( ! m_PendingStillWorking ) {

                    /* Nachricht posten, um anliegende Kommandos zu verarbeiten */

                    m_UpdateHandler.post( m_UpdateResults );
                }
            }
        }
    }

    /**
     * GUI-Update-Runnable
     */

    final Runnable m_UpdateResults = new Runnable() {

        /**
         * Methode, die aufgerufen wird, wenn der
         * GUI-Timer-Task, eine Nachricht postet. */

        public void run() {

            /* Flag testen, dass signalisiert, das der GUI-Updatehandler luft. */
            
            if ( m_PendingStillWorking ) {
                return;
            }

            /* Flag setzen, dass signalisiert, das der GUI-Updatehandler luft. */
            
            m_PendingStillWorking = true;
            
            /* Verzeichnisliste aktualisieren. */

            if ( m_UpdateFileList ) {
                m_UpdateFileList = false;

                getDir ( m_CurrentPath );
            }
            
            /* Flag zurcksetzen, dass signalisiert, das der GUI-Updatehandler luft. */

            m_PendingStillWorking = false;
        }
    };

    /**
     * Methode: Wird aufgerufen, um ein Verzeichnis samt
     *          Unterverzeichnisse zu lschen.
     * 
     * @param File fileobj  Dateiobjekt (Datei/Ordner)
     * 
     * @return boolean  true/false  Erfolg/Fehler
     */

    boolean DeleteRecursive( File fileobj ) {

        /* Inhalt rekursiv lschen. */

        if ( fileobj.isDirectory() ) {
            for ( File file : fileobj.listFiles() ) {

                /* Abbruch? */

                if ( m_CancelDeleteFlag ) {
                    return ( true );
                }

                DeleteRecursive( file );
            }
        }

        /* Objekt selber lschen */

        return ( fileobj.delete() );
    }

    /**
     * Methode: Wird aufgerufen, um eine im Listview ausgewhlte
     *          Datei/Ordner umzubenennen.
     * 
     * @param long id  Id des Listviewitems.
     */

    private void RenameFile( long id ) {
        String FileName;

        FileName = m_Path.get( ( int ) id );

        if ( FileName != null ) {

            /* Namen der umzubenennenden Datei ablegen. */

            m_RenameFile = FileName;

            /* Dialog zur Namenseingabe ffnen lassen. */

            EnterFileName( new File ( FileName ).isDirectory(), ( String ) m_List.get( ( int ) id ).get( ITEM_KEY ) );
        }
    }

    /**
     * Methode: Wird aufgerufen, um eine Ordnerebene nach oben zu wechseln.
     */

    private void OpenParentFolder() {
        final int position = 0;

        File file = new File( m_Path.get( position ) );

        /* Prfen, ob der Inhalt des Verzeichnisses gelesen werden darf. */

        if ( file.canRead() ) {

            /* Verzeichniswechsel auf hhere Ebene. */

            m_lastPositions.put( m_CurrentPath, position );

            getDir( m_Path.get( position ) );

            m_selectedFile = file;
        }
    }

}

/* EoF */
